@@ -21,24 +21,27 @@ import subprocess
 import sys
 
 if '.' not in __package__:
+  from common import sqlite3_logica
   from compiler import functors
   from compiler import rule_translate
   from compiler import universe
   from parser_py import parse
 else:
+  from ..common import sqlite3_logica
   from ..compiler import functors
   from ..compiler import rule_translate
   from ..compiler import universe
   from ..parser_py import parse
 
 
-def ParseOrExit(filename):
+def ParseOrExit(filename, import_root=None):
   """Parse a Logica program."""
   with open(filename) as f:
     program_text = f.read()
 
   try:
-    parsed_rules = parse.ParseFile(program_text)['rule']
+    parsed_rules = parse.ParseFile(program_text,
+                                   import_root=import_root)['rule']
   except parse.ParsingException as parsing_exception:
     parsing_exception.ShowMessage()
     sys.exit(1)
@@ -46,9 +49,9 @@ def ParseOrExit(filename):
   return parsed_rules
 
 
-def GetProgramOrExit(filename, user_flags=None):
+def GetProgramOrExit(filename, user_flags=None, import_root=None):
   """Get program object from a file."""
-  parsed_rules = ParseOrExit(filename)
+  parsed_rules = ParseOrExit(filename, import_root=import_root)
   try:
     p = universe.LogicaProgram(parsed_rules, user_flags=user_flags)
   except rule_translate.RuleCompileException as rule_compilation_exception:
@@ -60,19 +63,39 @@ def GetProgramOrExit(filename, user_flags=None):
   return p
 
 
-def RunQuery(sql, output_format='pretty', engine='bigquery'):
+def RunQuery(sql,
+             settings=None,
+             output_format='pretty', engine='bigquery'):
   """Run a SQL query on BigQuery."""
+  settings = settings or {}
   if engine == 'bigquery':
     p = subprocess.Popen(['bq', 'query',
                           '--use_legacy_sql=false',
                           '--format=%s' % output_format],
                          stdin=subprocess.PIPE, stdout=subprocess.PIPE)
   elif engine == 'sqlite':
-    p = subprocess.Popen(['sqlite3'],
-                         stdin=subprocess.PIPE, stdout=subprocess.PIPE)
+    # TODO: Make multi-statement scripts work.
+    return sqlite3_logica.RunSQL(sql)
   elif engine == 'psql':
     p = subprocess.Popen(['psql', '--quiet'],
                          stdin=subprocess.PIPE, stdout=subprocess.PIPE)
+  elif engine == 'trino':
+    catalog = settings.get('catalog', 'memory')
+    server = settings.get('server', 'http://localhost:8080')
+    p = subprocess.Popen(['trino',
+                          '--catalog=%s' % catalog,
+                          '--server=%s' % server] +
+                          ['--output-format=ALIGNED'],
+                          stdin=subprocess.PIPE, stdout=subprocess.PIPE)
+  elif engine == 'presto':
+    catalog = settings.get('catalog', 'memory')
+    server = settings.get('server', 'http://localhost:8080')
+    p = subprocess.Popen(['presto',
+                          '--catalog=%s' % catalog,
+                          '--server=%s' % server,
+                          '--file=/dev/stdin'] +
+                          ['--output-format=ALIGNED'],
+                          stdin=subprocess.PIPE, stdout=subprocess.PIPE)
   else:
     assert False, 'Unknown engine: %s' % engine
   o, _ = p.communicate(sql.encode())
@@ -80,8 +103,17 @@ def RunQuery(sql, output_format='pretty', engine='bigquery'):
 
 
 def RunPredicate(filename, predicate,
-                 output_format='pretty', user_flags=None):
+                 output_format='pretty', user_flags=None,
+                 import_root=None):
   """Run a predicate on BigQuery."""
-  p = GetProgramOrExit(filename, user_flags=user_flags)
+  p = GetProgramOrExit(filename, user_flags=user_flags,
+                       import_root=import_root)
   sql = p.FormattedPredicateSql(predicate)
-  return RunQuery(sql, output_format, engine=p.annotations.Engine())
+  engine = p.annotations.Engine()
+  if ('@Engine' in p.annotations.annotations and
+      engine in p.annotations.annotations['@Engine']):
+    settings = p.annotations.annotations['@Engine'][engine]
+  else:
+    settings = {}
+  return RunQuery(sql, settings,
+                  output_format, engine=engine)
