@@ -72,7 +72,7 @@ class HeritageAwareString(str):
     if stop > len(self):
       stop = len(self)
     if stop < 0:
-      stop = len(self) - stop
+      stop = len(self) + stop
     substring.start = self.start + start
     substring.stop = self.start + stop
     substring.heritage = self.heritage
@@ -268,9 +268,9 @@ def Strip(s):
 def StripSpaces(s):
   left_idx = 0
   right_idx = len(s) - 1
-  while left_idx < len(s) and s[left_idx] in [' ', '\n']:
+  while left_idx < len(s) and s[left_idx].isspace():
     left_idx += 1
-  while right_idx > left_idx and s[right_idx] in [' ', '\n']:
+  while right_idx > left_idx and s[right_idx].isspace():
     right_idx -= 1
   return s[left_idx:right_idx + 1]
 
@@ -533,6 +533,7 @@ def ParseList(s):
 
 def ParsePredicateLiteral(s):
   if (s == '++?' or
+      s == 'nil' or
       s and
       set(s) <=
       set(string.ascii_letters) | set(string.digits) | set(['_']) and
@@ -562,11 +563,13 @@ def ParseLiteral(s):
     return {'the_predicate': v}
 
 
-def ParseInfix(s):
+def ParseInfix(s, operators=None):
   """Parses an infix operator expression."""
+  operators = operators or [
+      '||', '&&', '->', '==', '<=', '>=', '<', '>', '!=',
+      '++?', '++', '+', '-', '*', '/', '%', '^', ' in ', '!']
   unary_operators = ['-', '!']
-  for op in ['||', '&&', '->', '==', '<=', '>=', '<', '>', '!=',
-             '++?', '++', '+', '-', '*', '/', '%', '^', ' in ', '!']:
+  for op in operators:
     parts = SplitRaw(s, op)
     if len(parts) > 1:
       # Right is the rightmost operand and left are all the other operands.
@@ -805,6 +808,9 @@ def ParseProposition(s):
     raise ParsingException('If-then-else clause is only supported as an '
                            'expression, not as a proposition.', s)
   c = ParseCall(s)
+  if c:
+    return {'predicate': c}
+  c = ParseInfix(s, operators=['&&', '||'])
   if c:
     return {'predicate': c}
   c = ParseUnification(s)
@@ -1093,10 +1099,9 @@ def SplitImport(import_str):
   return ('.'.join(import_parts[:-1]), import_parts[-1], synonym)
 
 
-def ParseImport(file_import_str, parsed_imports, import_chain):
+def ParseImport(file_import_str, parsed_imports, import_chain, import_root):
   """Parses an import, returns extracted rules."""
   file_import_parts = file_import_str.split('.')
-  file_path = '/'.join(file_import_parts) + '.l'
   if file_import_str in parsed_imports:
     if parsed_imports[file_import_str] is None:
       raise ParsingException(
@@ -1105,10 +1110,28 @@ def ParseImport(file_import_str, parsed_imports, import_chain):
           HeritageAwareString(file_import_str))    
     return None
   parsed_imports[file_import_str] = None
-  if not os.path.exists(file_path):
-    raise ParsingException('Imported file not found: %s.' % file_path,
-                           HeritageAwareString(file_import_str))
-  file_content = open(file_path).read()
+  if isinstance(import_root, str):
+    file_path = os.path.join(import_root, '/'.join(file_import_parts) + '.l')
+    if not os.path.exists(file_path):
+      raise ParsingException('Imported file not found: %s.' % file_path,
+                             HeritageAwareString(file_import_str))
+  else:
+    assert isinstance(import_root, list), 'import_root must be of type str or list.'
+    considered_files = []
+    for root in import_root:
+      file_path = os.path.join(root, '/'.join(file_import_parts) + '.l')
+      considered_files.append(file_path)
+      if os.path.exists(file_path):
+        break
+    else:
+      raise ParsingException(
+          'Imported file not found. Considered: \n- %s.' % '\n- '.join(
+              considered_files),
+          HeritageAwareString(
+              'import ' + file_import_str + '.<PREDICATE>')[7:-11])
+
+  with open(file_path) as f:
+    file_content = f.read()
   parsed_file = ParseFile(file_content, file_import_str, parsed_imports,
                           import_chain)
   parsed_imports[file_import_str] = parsed_file
@@ -1180,8 +1203,10 @@ class MultiBodyAggregation(object):
         n for n, rs in defined_predicates_rules.items()
         if len(rs) > 1 and 'distinct_denoted' in rs[0]]
     aggregation_field_values_per_predicate = {}
+    original_full_text_per_predicate = {}
     for rule in rules:
       name = rule['head']['predicate_name']
+      original_full_text_per_predicate[name] = rule['full_text']
       if name in multi_body_aggregating_predicates:
         aggregation, new_rule = cls.SplitAggregation(rule)
         if name in aggregation_field_values_per_predicate:
@@ -1230,7 +1255,7 @@ class MultiBodyAggregation(object):
                   }]
               }
           },
-          'full_text': 'automatically built',
+          'full_text': original_full_text_per_predicate[name],
           'distinct_denoted': True
       }
       new_rules.append(aggregating_rule)
@@ -1411,13 +1436,15 @@ class AggergationsAsExpressions(object):
     return rules
 
 
-def ParseFile(s, this_file_name=None, parsed_imports=None, import_chain=None):
+def ParseFile(s, this_file_name=None, parsed_imports=None, import_chain=None,
+              import_root=None):
   """Parsing logica.Logica."""
   s = HeritageAwareString(RemoveComments(HeritageAwareString(s)))
   parsed_imports = parsed_imports or {}
   this_file_name = this_file_name or 'main'
   import_chain = import_chain or []
   import_chain = import_chain + [this_file_name]
+  import_root = import_root or ''
   str_statements = Split(s, ';')
   rules = []
   imported_predicates = []
@@ -1428,7 +1455,8 @@ def ParseFile(s, this_file_name=None, parsed_imports=None, import_chain=None):
     if str_statement.startswith('import '):
       import_str = str_statement[len('import '):]
       file_import_str, import_predicate, synonym = SplitImport(import_str)
-      ParseImport(file_import_str, parsed_imports, import_chain)
+      ParseImport(file_import_str, parsed_imports, import_chain,
+                  import_root)
       imported_predicates.append({
           'file': file_import_str, 'predicate_name': import_predicate,
           'synonym': synonym})
